/*
  Copyright 2009 Patrick Ahlbrecht

   Licensed under the Apache License, Version 2.0 (the "License");
   you may not use this file except in compliance with the License.
   You may obtain a copy of the License at

       http://www.apache.org/licenses/LICENSE-2.0

   Unless required by applicable law or agreed to in writing, software
   distributed under the License is distributed on an "AS IS" BASIS,
   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
   See the License for the specific language governing permissions and
   limitations under the License.
*/

import java.awt.*;

/**
 * One cell in the table or "bag" in the layout. This class is more or less
 * a data container.
 */
public class Cell {

  /**
   * Constraints as translated from the HTML table
   */
  public GridBagConstraints gbc;
  
  /**
   * Name of the component to put in this cell
   */
  public String compName;
  
  /**
   * Construct a new cell/bag
   * @param compName name of the component to put into this cell
   * @param gbc constraints for the cell
   */
  public Cell(String compName, GridBagConstraints gbc) {
    this.compName=compName;
    this.gbc = gbc;
  }
  
  /**
   * Convinience function to turn an array of cells into ready to use code
   * @param cells the cells to include in the code
   * @param indent what to indent the code with
   * @param lname name of the <code>GridBagLayout</code> object (e.g. gbl)
   * @param cname name of the <code>GridBagConstrains</code> object (e.g. gbc)
   * @param container name of the container object to layout (or null for this)
   * @return Java code, that can be copyed and pasted into the final source.
   */
  public static String generate(Cell[] cells, String indent, String lname, String cname, String container) {
    StringBuilder sb  = new StringBuilder();
    sb.append(indent+"GridBagLayout "+lname+" = new GridBagLayout();\n");
    if (container!=null) sb.append(indent+container+".");
    else sb.append(indent);
    sb.append("setLayout("+lname+");\n");
    sb.append(indent+"GridBagConstraints "+cname+" = new GridBagConstraints();\n");
    for (Cell cell:cells) sb.append(cell.toJava(indent,lname,cname,container));
    return sb.toString();
  }
  
  /**
   * Transform this Cell into a sniplet of Java code
   * @param indent what to indent the code with
   * @param lname name of the <code>GridBagLayout</code> object (e.g. gbl)
   * @param cname name of the <code>GridBagConstrains</code> object (e.g. gbc)
   * @param container name of the container object to layout (or null for this)
   * @return Java code, that can be copyed and pasted into the final source.
   */
  public String toJava(String indent, String lname, String cname, String container) {
    StringBuilder sb = new StringBuilder("\n"+indent+"// Component: "+compName+"\n");
    sb.append(indent+cname+".gridx = "+toSymbol("gridx",gbc.gridx)+";\n");
    sb.append(indent+cname+".gridy = "+toSymbol("gridy",gbc.gridy)+";\n");
    sb.append(indent+cname+".gridwidth = "+gbc.gridwidth+";\n");
    sb.append(indent+cname+".gridheight = "+gbc.gridheight+";\n");
    sb.append(indent+cname+".weightx = "+gbc.weightx+";\n");
    sb.append(indent+cname+".weighty = "+gbc.weighty+";\n");
    sb.append(indent+cname+".anchor = "+toSymbol("anchor",gbc.anchor)+";\n");
    sb.append(indent+cname+".fill = "+toSymbol("fill",gbc.fill)+";\n");
    sb.append(indent+cname+".ipadx = "+gbc.ipadx+";\n");
    sb.append(indent+cname+".ipady = "+gbc.ipady+";\n");
    if (gbc.insets!=null) {
      sb.append(indent+cname+".insets = new Insets(");
      sb.append(gbc.insets.top+",");
      sb.append(gbc.insets.left+",");
      sb.append(gbc.insets.bottom+",");
      sb.append(gbc.insets.right+");\n");
    }
    sb.append(indent+lname+".setConstraints("+compName+","+cname+");\n");
    if (container!=null) sb.append(indent+container+".");
    else sb.append(indent);
    sb.append("add("+compName+");\n");
    return sb.toString();
  }
  
  /**
   * Helper function for translating numerical constraints back to symbolic
   * constants.
   * @param name name of the constraint
   * @param val numerical value
   * @return symbolic value or vval as string, if no symbol could be found
   */
  private String toSymbol(String name, int val) {
    String ret = ""+val;
    
    if (name.equals("anchor")) {
      switch(val) {
        case GridBagConstraints.NORTHWEST: {
          ret = "GridBagConstraints.NORTHWEST";
          break;
        }
      }
      switch(val) {
        case GridBagConstraints.NORTH: {
          ret = "GridBagConstraints.NORTH";
          break;
        }
      }
      switch(val) {
        case GridBagConstraints.NORTHEAST: {
          ret = "GridBagConstraints.NORTHEAST";
          break;
        }
      }
      switch(val) {
        case GridBagConstraints.WEST: {
          ret = "GridBagConstraints.WEST";
          break;
        }
      }
      switch(val) {
        case GridBagConstraints.CENTER: {
          ret = "GridBagConstraints.CENTER";
          break;
        }
      }
      switch(val) {
        case GridBagConstraints.EAST: {
          ret = "GridBagConstraints.EAST";
          break;
        }
      }
      switch(val) {
        case GridBagConstraints.SOUTHWEST: {
          ret = "GridBagConstraints.SOUTHWEST";
          break;
        }
      }
      switch(val) {
        case GridBagConstraints.SOUTH: {
          ret = "GridBagConstraints.SOUTH";
          break;
        }
      }
      switch(val) {
        case GridBagConstraints.SOUTHEAST: {
          ret = "GridBagConstraints.SOUTHEAST";
          break;
        }
      }
    }
    
    if (name.equals("fill")) {
      switch(val) {
        case GridBagConstraints.NONE: {
          ret = "GridBagConstraints.NONE";
          break;
        }
        case GridBagConstraints.BOTH: {
          ret = "GridBagConstraints.BOTH";
          break;
        }
        case GridBagConstraints.HORIZONTAL: {
          ret = "GridBagConstraints.HORIZONTAL";
          break;
        }
        case GridBagConstraints.VERTICAL: {
          ret = "GridBagConstraints.VERTICAL";
          break;
        }
      }
    }
    
    return ret;
  }
}